"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnergySocketAccessory = void 0;
const types_1 = require("./api/types");
class EnergySocketAccessory {
    constructor(platform, accessory, api) {
        this.platform = platform;
        this.accessory = accessory;
        const properties = accessory.context.energySocket;
        this.properties = properties;
        const loggerPrefix = `[Energy Socket: ${properties.displayName}] -> `;
        this.loggerPrefix = loggerPrefix;
        this.platform.log.debug(this.loggerPrefix, `Initializing platform accessory ${JSON.stringify(properties)}`);
        this.homeWizardApi = api;
        const informationService = this.accessory.getService(this.platform.Service.AccessoryInformation);
        informationService === null || informationService === void 0 ? void 0 : informationService.setCharacteristic(this.platform.Characteristic.Manufacturer, types_1.PLATFORM_MANUFACTURER);
        informationService === null || informationService === void 0 ? void 0 : informationService.setCharacteristic(this.platform.Characteristic.Model, this.getModel(properties.productName, properties.productType));
        informationService === null || informationService === void 0 ? void 0 : informationService.setCharacteristic(this.platform.Characteristic.SerialNumber, properties.serialNumber);
        informationService === null || informationService === void 0 ? void 0 : informationService.setCharacteristic(this.platform.Characteristic.FirmwareRevision, properties.firmwareVersion);
        this.informationService = informationService;
        this.service =
            this.accessory.getService(this.platform.Service.Outlet) ||
                this.accessory.addService(this.platform.Service.Outlet);
        this.service.setCharacteristic(this.platform.Characteristic.Name, properties.displayName);
        this.service.setCharacteristic(this.platform.Characteristic.OutletInUse, true);
        this.service
            .getCharacteristic(this.platform.Characteristic.On)
            .onSet(this.handleSetOn.bind(this))
            .onGet(this.handleGetOn.bind(this));
        this.accessory.on("identify", this.handleIdentify.bind(this));
    }
    get firmwareVersion() {
        var _a;
        const firmwareVersionString = (_a = this.informationService) === null || _a === void 0 ? void 0 : _a.getCharacteristic(this.platform.Characteristic.FirmwareRevision).value;
        const firmwareVersion = firmwareVersionString ? Number(firmwareVersionString) : null;
        return firmwareVersion;
    }
    handleIdentify() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.homeWizardApi.putIdentify(this.firmwareVersion);
                return response;
            }
            catch (error) {
                const fallbackErrorMessage = 'A unknown error occurred while identifying the Energy Socket';
                throw this.handleAccessoryApiError(error, fallbackErrorMessage);
            }
        });
    }
    getModel(productName, productType) {
        return `${productName} (${productType})`;
    }
    get isSwitchLockEnabled() {
        var _a;
        return ((_a = this.localStateResponse) === null || _a === void 0 ? void 0 : _a.switch_lock) === true;
    }
    handleSetOn(value) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (this.isSwitchLockEnabled) {
                    this.platform.log.warn(this.loggerPrefix, `This Energy Socket (${this.properties.serialNumber}) is locked. Please enable the "Switch lock" setting in the HomeWizard Energy app for this Energy Socket.`);
                    throw new this.platform.api.hap.HapStatusError(-70412);
                }
                const response = yield this.homeWizardApi.putState({
                    power_on: value,
                });
                this.platform.log.info(this.loggerPrefix, `Energy Socket state is updated to ${response.power_on ? 'ON' : 'OFF'}`);
            }
            catch (error) {
                const fallbackErrorMessage = 'A unknown error occurred while setting the ON state';
                throw this.handleAccessoryApiError(error, fallbackErrorMessage);
            }
        });
    }
    setLocalStateResponse(response) {
        this.localStateResponse = response;
    }
    handleGetOn() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield this.homeWizardApi.getState();
                this.setLocalStateResponse(response);
                this.platform.log.info(this.loggerPrefix, `Energy Socket state is ${response.power_on ? 'ON' : 'OFF'}`);
                return response.power_on;
            }
            catch (error) {
                const errorMessage = 'A unknown error occurred while getting the ON state';
                throw this.handleAccessoryApiError(error, errorMessage);
            }
        });
    }
    handleAccessoryApiError(error, fallbackErrorMessage) {
        let errorMessage = fallbackErrorMessage || 'A unknown error occurred';
        if (error instanceof Error) {
            errorMessage = error.message;
        }
        this.platform.log.error(this.loggerPrefix, errorMessage);
        return new this.platform.api.hap.HapStatusError(-70402);
    }
}
exports.EnergySocketAccessory = EnergySocketAccessory;
//# sourceMappingURL=energy-socket-accessory.js.map